@@ -82,13 +82,13 @@
 
   if (is.matrix(unpenalized)) {
     if (ncol(unpenalized) > 0) 
-      startgamma <- coefficients(coxph(response ~ ., data = as.data.frame(unpenalized)))
+      startgamma <- coefficients(coxph(response ~ ., data = as.data.frame(unpenalized), method = "breslow"))
     else
       startgamma <- numeric(0)
   } else {
     .response <- response
     form <- as.formula(paste(".response~", paste(c("1", attr(terms(unpenalized),"term.labels")), collapse="+"))) 
-    startgamma <- coefficients(coxph(form, data = data))
+    startgamma <- coefficients(coxph(form, data = data, method = "breslow"))
   }
 }
 
